local L = L_MinHud
local FN = L.Functions

local FU = FRACUNIT

FN.DoCTFFlag = function(v, x, y, flags, team)
-- Shows the status of the flag in question.
	for player in players.iterate do
		if not (player and player.valid) then continue end -- Skip if we have no valid player.
		if not (player.gotflag) then continue end -- Must have a flag.
		
		-- Is flag at base?
		if (team == 1) and (player.gotflag & GF_REDFLAG) then
			v.drawScaled(x*FU, y*FU, FU/2, v.cachePatch("HUD_RFLAGTAKEN"), flags)
			break
		elseif (team == 2) and (player.gotflag & GF_BLUEFLAG) then
			v.drawScaled(x*FU, y*FU, FU/2, v.cachePatch("HUD_BFLAGTAKEN"), flags)
			break
		end
	end
	
	-- How long till the flag returns on it's own?
	if (team == 1) and (L.RedFlag and L.RedFlag.valid) and (L.RedFlag.fuse > 1) then
		v.drawString(x, y + 4, L.RedFlag.fuse/TICRATE, flags|V_YELLOWMAP, "thin-center")
	elseif (team == 2) and (L.BlueFlag and L.BlueFlag.valid) and (L.BlueFlag.fuse > 1)
		v.drawString(x, y + 4, L.BlueFlag.fuse/TICRATE, flags|V_YELLOWMAP, "thin-center")
	end
end

local xoffs, yoffs = FixedMul(FU/2, FU/2), FixedMul(FU*14 + FU/2, FU/2)
FN.DoCTFCompass = function(v, player, cam, x, y, flags, trackmobj, team, compasscolor)
-- Draw compass pointing at specified object.
-- If specified, point towards player carrying object. (CTF-specific)
	local player = displayplayer
	local validplayer = (player and player.valid) and (player.realmo and player.realmo.valid) and true or false
	local track = trackmobj
	local zcompass_x, zcompass_y = 0, 0
	
	-- Gather coordinates. Needed for compass' angle math.
	local xx, yy, zz = cam.x, cam.y, cam.z
	local lookang = cam.angle
	local ourz, zdiff = (cam.z + cam.height/2), (cam.height*14)
	if (validplayer) then
		xx, yy, zz = player.realmo.x, player.realmo.y, player.realmo.z
		lookang = player.cmd.angleturn<<16
		ourz, zdiff = (player.realmo.z + player.realmo.height/2), (224*player.realmo.scale)
	end
	
	for trackplayer in players.iterate
		-- Track according to the team specified.
		-- This is done to get around an issue.
		if (trackplayer and trackplayer.valid)
		and (trackplayer.mo and trackplayer.mo.valid)
		and (trackplayer.gotflag)
			if (trackplayer.ctfteam == 1)
			and (trackplayer.gotflag == 1)
			and (team == 1)
				track = trackplayer.mo
			elseif (trackplayer.ctfteam == 2)
			and (trackplayer.gotflag == 2)
			and (team == 2)
				track = trackplayer.mo
			end
		end
	end
	
	if (track and track.valid) and not (player.mo and player.mo == track) then
		local angle
		if twodlevel then
			-- Account for 2D level when pointing.
			angle = R_PointToAngle2(xx, zz, track.x, track.z) - ANGLE_90
		else
			-- Point as normal.
			angle = R_PointToAngle2(xx, yy, track.x, track.y) - lookang
		end
		local xpoint = P_ReturnThrustX(nil, angle - ANGLE_90, 10*FU)
		local ypoint = P_ReturnThrustY(nil, angle - ANGLE_90, 10*FU)
		local xycompass = v.getSpritePatch("LCKN", E, 0, angle)
		
		v.drawScaled(x*FU + xoffs - xpoint, y*FU + yoffs + ypoint, FU/2, xycompass, flags, compasscolor)
	end
end