local L = L_MinHud
local FN = L.Functions

FN.GetSkin = function(player)
-- Return skin integer alongside skin table data.
	if (player and player.valid) then
		return player.skin, skins[player.skin]
	end
	
	return 0, skins[0]
end

FN.GetColor = function(v, player, colorize, spectating)
-- Return player color.
-- If specified... return TC_RAINBOW, if player.mo is colorized.
-- If specified... return a unique skincolor used specifically for spectators.
	local palette, color = TC_DEFAULT, SKINCOLOR_GREEN
	
	if (spectating) and (player.spectator) then
		color = SKINCOLOR_CLOUDY
	elseif (player.realmo and player.realmo.valid) then
		color = player.realmo.color
	elseif (player.skincolor) then
		color = player.skincolor
	end
	
	if (colorize) and (player.realmo and player.realmo.valid) and (player.realmo.colorized) then
		palette = TC_RAINBOW
	else
		palette = FN.GetSkin(player)
	end
	
	return v.getColormap(palette, color)
end

FN.GetTransparency = function(player, independent)
-- Return transparency flag. Half if player is spectating, full if not.
-- If specified, do not care for local player's HUD transparency setting.
	if (player.spectator) then
		if (independent)
			return V_50TRANS
		else
			return V_HUDTRANSHALF
		end
	else
		if (independent)
			return 0
		else
			return V_HUDTRANS
		end
	end
end

FN.GetName = function(player)
-- Return character name. Will be colored depending on the situation.
	local name, ogname = skins[0].hudname, skins[0].hudname
	local color = "\x80"
	if (player and player.valid) then
		name = skins[player.skin].hudname
		ogname = skins[player.skin].hudname
	end
	
	-- Color the name accordingly to the player.
	if (player.spectator) then
		color = "\x86" -- Gray.
	elseif (player.pflags & PF_TAGIT) then
		color = "\x87" -- Orange.
	elseif (gametyperules & GTR_TEAMS and G_GametypeHasTeams and player.ctfteam == 1) then
		color = "\x85" -- Red.
	elseif (gametyperules & GTR_TEAMS and G_GametypeHasTeams and player.ctfteam == 2) then
		color = "\x84" -- Blue.
	else
		color = "\x82" -- Yellow.
	end
	
	-- Give color to the name.
	name = color+$
	
	return name, ogname
end

local FixedAlignment = {
	["left"] = "fixed",
	["thin"] = "thin-center",
	["small"] = "small-fixed",
	["small-thin"] = "small-thin-fixed",
}
local AlignToCenter = {
	["left"] = "center",
	["fixed"] = "fixed-center",
	["thin"] = "thin-center",
	["thin-fixed"] = "thin-fixed-center",
	["small"] = "small-center",
	["small-fixed"] = "small-fixed-center",
	["small-thin"] = "small-thin-center",
	["small-thin-fixed"] = "small-thin-fixed-center",
}
local AlignToRight = {
	["left"] = "right",
	["fixed"] = "fixed-right",
	["thin"] = "thin-right",
	["thin-fixed"] = "thin-fixed-right",
	["small"] = "small-right",
	["small-fixed"] = "small-fixed-right",
	["small-thin"] = "small-thin-right",
	["small-thin-fixed"] = "small-thin-fixed-right",
}
FN.ShortenName = function(v, name, alignment)
	-- Shorten the name if it's too long.
	local width
	if string.len(name) <= 5 or v.stringWidth(name) <= 48 then
		width = "left"
	elseif v.stringWidth(name, 0, "thin") <= 40 then
		width = "thin"
	else
		width = "small-thin"
	end
	
	-- If the alignment we're given is of a 'fixed' type, then convert it to the equivalent!
	if (alignment == "fixed") or (alignment == "fixed-center") or (alignment == "fixed-right") then
		width = FixedAlignment[width]
	end
	
	-- If the alignment we're given has 'center' or 'right', then convert to the equivalent of whichever we we're given.
	if (alignment == "center") or (alignment == "fixed-center") then
		width = AlignToCenter[width]
	elseif (alignment == "right") or (alignment == "fixed-right") then
		width = AlignToRight[width]
	end
	
	return width
end

FN.GetLives = function(player)
-- Return lives string/integer based on player and game state.
	if not G_GametypeUsesLives() then
	-- Return string based on team affiliation, if lives aren't used.
		if G_IsSpecialStage() then -- Not in a Special Stage...
			return ''
		elseif (player.pflags & PF_TAGIT) then -- Tagger!
			return 'IT!'
		elseif (gametyperules & GTR_TEAMS and G_GametypeHasTeams and player.ctfteam == 1) then -- On red team!
			return 'RED'
		elseif (gametyperules & GTR_TEAMS and G_GametypeHasTeams and player.ctfteam == 2) then -- On blue team!
			return 'BLUE'
		else
			return ''
		end
	-- Return string for lives if they're used.
	elseif G_GametypeUsesLives() and G_GametypeUsesCoopLives() and CV_FindVar("cooplives").value == 0 and (netgame or multiplayer)
	or G_PlatformGametype() and not (gametyperules & GTR_LIVES)
		then return '\x16' -- Infinity symbol. Can't die with never-ending lives, can you?
	elseif G_GametypeUsesLives() or (gametyperules & GTR_LIVES)
		then return tostring(player.lives) -- Lives. Not much to say.
	else
		return '' -- Failsafe.
	end
end