local L = L_MinHud
local FN = L.Functions

local FU = FRACUNIT

-- Pointer to graphics.
local none, pity, pink = 'SH_NONE', 'SH_PITY', 'SH_PINK'
local wind, force, elem, magnet, nuke = 'SH_WHIRLWIND', 'SH_FORCE', 'SH_ELEMENTAL', 'SH_ATTRACTION', 'SH_ARMAGEDDON'
local water, fire, elec = 'SH_BUBBLE', 'SH_FLAME', 'SH_THUNDER'

FN.GetFlagPatch = function(v, player)
-- Return patch based on flag being carried.
	local patch = 'HUD_BLANK'
	
	if (player.gotflag == 1) then
		patch = 'PW_RFLAG'
	elseif (player.gotflag == 2) then
		patch = 'PW_BFLAG'
	end
	
	return v.cachePatch(patch)
end

FN.GetShieldPatch = function(v, shield)
-- Return patch to use based on shield player has.
	local patch
	
	if (shield == SH_PITY) then
		patch = pity
	elseif shield == SH_PINK then
		patch = pink
	
	elseif (shield == SH_WHIRLWIND) then
		patch = wind
	elseif (shield & SH_FORCE)
		if (shield & SH_FORCEHP) then
			patch = force
		else
			patch = (L.Tic/2 & 1) and force or none
		end
	elseif (shield == SH_ELEMENTAL) then
		patch = elem
	elseif (shield == SH_ATTRACT) then
		patch = magnet
	elseif (shield == SH_ARMAGEDDON) then
		patch = nuke
	
	elseif (shield == SH_BUBBLEWRAP) then
		patch = water
	elseif (shield == SH_FLAMEAURA) then
		patch = fire
	elseif (shield == SH_THUNDERCOIN) then
		patch = elec
	
	else
		patch = none
	end
	
	return v.cachePatch(patch)
end

FN.GetStarsPatch = function(v, player)
-- Return patch and timer for invulnerability.
-- Do 'flash' of the patch if the power up is about to end.
	local tics = max(0, max(player.powers[pw_flashing], player.powers[pw_invulnerability]))
	local patch = 'PW_STARS'+abs(P_ReturnThrustX(nil, FixedAngle( (L.Tic/2%9 * 22) * FU), 4))
	
	if (tics <= TICRATE*3) and (L.Tic%2==0)
		patch = 'HUD_BLANK'
	end
	
	return v.cachePatch(patch), tics
end

FN.GetShoesPatch = function(v, player)
-- As the invulnerability function, but super sneakers.
	local tics = max(0, player.powers[pw_sneakers])
	local patch = 'PW_SHOES'
	
	if (tics <= TICRATE*3) and (L.Tic%2==0)
		patch = 'HUD_BLANK'
	end
	
	return v.cachePatch(patch), tics
end

FN.GetBootsPatch = function(v, player)
-- As the invulnerability function, but gravity boots.
	local tics = max(0, player.powers[pw_gravityboots])
	local patch = 'PW_GRAV'
	
	if (tics <= TICRATE*3) and (L.Tic%2==0)
		patch = 'HUD_BLANK'
	end
	
	return v.cachePatch(patch), tics
end

local lerp0 = {
	["FINISH"] = 0,
	["FLAG"] = 0,
	["SHIELD"] = 0,
	["RESERVE"] = 0,
	["IFRAMES"] = 0,
	["SHOES"] = 0,
	["BOOTS"] = 0,
}
local lerp1 = {
	["FINISH"] = 0,
	["FLAG"] = 0,
	["SHIELD"] = 0,
	["RESERVE"] = 0,
	["IFRAMES"] = 0,
	["SHOES"] = 0,
	["BOOTS"] = 0,
}
FN.DrawPowerups = function(v, player, x, y, scale, flags, sx, sy, salign, dist, angle)
-- Display all current *powerups obtained by the player. This function handles most of it.
	local lerp = 0
	if (player == displayplayer) then
		lerp = lerp0
	else
		lerp = lerp1
	end
	
	local lerpx, lerpy = 0, 0
	local function DoShift(val, ang)
		lerpx, lerpy = $+P_ReturnThrustX(nil, ang, val), $+P_ReturnThrustY(nil, ang, val)
	end
	local function DoLerp(kind, retract, val)
		if (player == displayplayer) then
			if (retract)
				lerp0[kind] = FN.FixedLerp(lerp0[kind], 0, FU*60/100)
			else
				lerp0[kind] = FN.FixedLerp(lerp0[kind], val, FU*60/100)
			end
		else
			if (retract)
				lerp1[kind] = FN.FixedLerp(lerp1[kind], 0, FU*60/100)
			else
				lerp1[kind] = FN.FixedLerp(lerp1[kind], val, FU*60/100)
			end
		end
	end
	
	local shield = (player.powers[pw_shield] & ~SH_FIREFLOWER)
	
	local ipatch, itics = FN.GetStarsPatch(v, player)
	local spatch, stics = FN.GetShoesPatch(v, player)
	local bpatch, btics = FN.GetBootsPatch(v, player)
	
	local bouncetics = (L.Tic%24)*15
	local bounce = abs(P_ReturnThrustY(nil, FixedAngle(bouncetics*FU), FU*4))
	bounce = FixedMul($, scale) -- We shouldn't be bouncing THAT high while small.
	
	if (player.pflags & PF_FINISHED) then -- FINISHED flag.
		v.drawScaled(x+lerpx, y+lerpy, scale, v.cachePatch('PW_FFLAG'), flags)
		
		DoShift(lerp["FINISH"], angle)
		DoLerp("FINISH", false, dist)
	else
		DoShift(lerp["FINISH"], angle)
		DoLerp("FINISH", true, dist)
	end
	
	if (player.gotflag) then -- CTF flag.
		v.drawScaled(x+lerpx, y+lerpy, scale, FN.GetFlagPatch(v, player), flags)
		
		DoShift(lerp["FLAG"], angle)
		DoLerp("FLAG", false, dist)
	else
		DoShift(lerp["FLAG"], angle)
		DoLerp("FLAG", true, dist)
	end
	
	if (shield) then -- Shield.
		v.drawScaled(x+lerpx, y+lerpy, scale, FN.GetShieldPatch(v, shield), flags)
		
		DoShift(lerp["SHIELD"], angle)
		DoLerp("SHIELD", false, dist)
	else
		DoShift(lerp["SHIELD"], angle)
		DoLerp("SHIELD", true, dist)
	end
	
	if (player.playerstate == PST_LIVE) and (player.shieldmax)
	and (gametyperules & GTR_PITYSHIELD) then -- BattleMod: Reserve shields.
		local xreserve, yreserve, patch = 0, 0, v.cachePatch('SH_NONE')
		local function drawshield(n)
			if player.shieldstock[n]
			and (not (P_PlayerInPain(player) or player.charmed) or #player.shieldstock > n or L.Tic&1) -- Reserve shield is about to be used (blinking).
				patch = FN.GetShieldPatch(v, player.shieldstock[n])
			end
			v.drawScaled(x+lerpx, y+lerpy, scale, patch, flags)
			DoShift(lerp["RESERVE"], angle)
			DoLerp("RESERVE", false, dist)
		end
		
		local n = player.shieldmax
		while n > 0 do
			drawshield(n)
			n = $-1
		end
	else
		DoShift(lerp["RESERVE"], angle)
		DoLerp("RESERVE", true, dist)
	end
	
	if (itics) then -- Invulnerability.
		v.drawScaled(x+lerpx, y+lerpy, scale, ipatch, flags)
		v.drawString(sx+lerpx, sy+lerpy, (itics/TICRATE), flags, salign)
		
		DoShift(lerp["IFRAMES"], angle)
		DoLerp("IFRAMES", false, dist)
	else
		DoShift(lerp["IFRAMES"], angle)
		DoLerp("IFRAMES", true, dist)
	end
	
	if (stics) then -- Super Sneakers.
		v.drawScaled(x+lerpx, y+lerpy - bounce, scale, spatch, flags)
		v.drawString(sx+lerpx, sy+lerpy, (stics/TICRATE), flags, salign)
		
		DoShift(lerp["SHOES"], angle)
		DoLerp("SHOES", false, dist)
	else
		DoShift(lerp["SHOES"], angle)
		DoLerp("SHOES", true, dist)
	end
	
	if (btics) then -- Gravity Boots.
		v.drawScaled(x+lerpx, y+lerpy, scale, bpatch, flags)
		v.drawString(sx+lerpx, sy+lerpy, (btics/TICRATE), flags, salign)
		
		DoShift(lerp["BOOTS"], angle)
		DoLerp("BOOTS", false, dist)
	else
		DoShift(lerp["BOOTS"], angle)
		DoLerp("BOOTS", true, dist)
	end
end

FN.DrawOxygenMeter = function(v, player, x, y, flags)
-- Draw player's oxygen meter.
	local patch = v.getSpritePatch(SPR_BBLS, L.Tic%24/6)
	local airtime
	
	if (player.powers[pw_underwater] and player.powers[pw_spacetime])
		-- If both air timers are active, use the air timer with the least time left.
		airtime = min(player.powers[pw_underwater], player.powers[pw_spacetime])
	else
		-- Use whichever one is active otherwise.
		airtime = (player.powers[pw_spacetime]) and player.powers[pw_spacetime] or player.powers[pw_underwater]
	end
	
	if (airtime < 1) then
		-- None active. Don't bother.
		return
	end
	
	if (player.playerstate == PST_DEAD) then
		-- Player drowned or died by other causes. Don't bother.
		return
	end
	
	if not (paused) and (airtime/TICRATE <= 10) and (L.Tic%35 == 0) then
		-- Play alarm sound if low on air. Unless the game is paused.
		S_StartSound(nil, sfx_cdfm49, nil)
	end
	
	if (airtime/TICRATE <= 10) and not (L.Tic%10 < 5) then
		-- Keep in line with the flashing from third person.
		return
	end
	
	v.draw(x-9, y+6, patch, flags)
	v.drawString(x+9, y, airtime/TICRATE, flags, 'center')
end