local L = L_MinHud
local CV = L.Console
local FN = L.Functions

FN.DoCountdownLogic = function(player)
-- Determine if countdown should be drawn. Return tics for purposes.
-- Manage variables related to countdown.
	if (player == displayplayer) then
		L.CountdownBounce = FN.FixedLerp(0, FU, $*80/100)
		if not (paused or L.CountdownLock) then L.CountdownTics = max(0, $-1) end
	end
	
	if (L.CountdownSound) and (L.CountdownSound ~= sfx_none) then
		if FN.Draw() and not FN.DrawSpecial() then S_StartSound(nil, L.CountdownSound, nil) end
		L.CountdownSound = sfx_none
	end
	
	local tics = -1
	local hidetime = CV_FindVar("hidetime").value*TICRATE
	local timelimit = CV_FindVar("timelimit").value*60*TICRATE
	local hidecountdown = hidetime - leveltime
	local timecountdown = timelimit - leveltime
	
	-- Gametype uses a time limit - countdown when it's about to end/enter overtime.
	if (gametyperules & GTR_TIMELIMIT)
	and (timecountdown > 0) and (timecountdown <= 3*TICRATE)
	and (timecountdown%TICRATE == 0)
		tics = max(-1, timelimit - leveltime)
		return true, tics
	end
	
	-- Gametype uses a starting countdown.
	if (gametyperules & GTR_STARTCOUNTDOWN)
	and (hidecountdown >= 0) and (hidecountdown <= 3*TICRATE)
	and (hidecountdown%TICRATE == 0)
		tics = max(-1, hidetime - leveltime)
		return true, tics
	end
	
	-- Gametype is a race - do a starting countdown.
	if (gametyperules & GTR_RACE)
	and (leveltime >= TICRATE and leveltime <= 4*TICRATE)
	and (leveltime%TICRATE == 0)
		tics = max(-1, TICRATE*4 - leveltime)
		return true, tics
	end
	
	return false, tics
end

FN.DoCountdown = function(v, player)
-- Do the "3, 2, 1, GO!". Works in conjunction with assisting functions.
	local racenum, sound, bounce = "HUD_BLANK", sfx_none, L.CountdownBounce
	local trigger, tics = FN.DoCountdownLogic(player)
	
	if (tics/TICRATE) == 3
		racenum = "HUD_RACE3"
		sound = sfx_s3ka7
	elseif (tics/TICRATE) == 2
		racenum = "HUD_RACE2"
		sound = sfx_s3ka7
	elseif (tics/TICRATE) == 1
		racenum = "HUD_RACE1"
		sound = sfx_s3ka7
	elseif (tics/TICRATE) == 0
		racenum = "HUD_RACEGO"
		sound = sfx_s3kad
	end
	
	if (trigger) -- Setup the timer, bounce, number to display, and sound to play.
	and not (L.CountdownLock)
		L.CountdownTics = TICRATE
		L.CountdownBounce = FU*8
		L.CountdownNumber = racenum
		L.CountdownSound = sound
		L.CountdownLock = true
	elseif not (trigger)
	and (L.CountdownLock)
		L.CountdownLock = false
	end
	
	if (L.CountdownTics) and FN.Draw() and not FN.DrawSpecial() then
		v.drawScaled(160*FU, 92*FU-L.CountdownBounce, FU, v.cachePatch(L.CountdownNumber), V_SNAPTOBOTTOM|V_PERPLAYER)
	end
end

FN.DoOvertime = function(v)
-- Draw the "Overtime!" when time limit is reached, but a stalemate is at play.
	local lt, tlimit = leveltime, CV_FindVar("timelimit").value
	local overtime = CV_FindVar("overtime").value
	
	if (gametyperules & GTR_TIMELIMIT) and (gametyperules & GTR_OVERTIME) and (overtime) then
		if (lt > 15*TICRATE) and (lt > (((tlimit*60)*TICRATE) + TICRATE/2)) and (tlimit) and (L.Tic/TICRATE%2 == 0)
			L.Overtime = true
		else
			L.Overtime = false
		end
		if FN.Draw() and not FN.DrawSpecial()
		and (lt > 15*TICRATE) and (lt == (tlimit*60)*TICRATE)
			then S_StartSound(nil, sfx_s3k63, nil)
		end
	end
	
	if (L.Overtime) and FN.Draw() and not FN.DrawSpecial() then
		v.drawString(160, 134, "OVERTIME!", V_SNAPTOBOTTOM|V_PERPLAYER, center)
	end
end

FN.DoTimerLogic = function(player)
-- Do the logic for the timer component.
-- Return if:
-- We flash or not? Done when doing a starting countdown or game is close to ending (time limit)
-- The time to use for displaying?
	local flash, tics = false
	
	local pt, lt = player.realtime, leveltime
	local puretlimit, purehlimit = CV_FindVar("timelimit").value, CV_FindVar("hidetime").value
	local tlimit = puretlimit * 60 * TICRATE
	local hlimit = purehlimit * TICRATE
	
	-- Counting down the hidetime?
	if (gametyperules & GTR_STARTCOUNTDOWN)
	and (pt <= hlimit)
		tics = hlimit - pt
		flash = true
	else
		-- Time limit?
		if (gametyperules & GTR_TIMELIMIT)
		and (puretlimit) and (tlimit > 0)
			if (tlimit > pt)
				tics = tlimit - pt
			else -- Overtime!
				tics = 0
			end
			flash = true
		-- Post-hidetime normal.
		elseif (gametyperules & GTR_STARTCOUNTDOWN)
			tics = tlimit - pt
		else
			tics = pt
		end
	end
	
	flash = (flash and (tics < 30*TICRATE) and (L.Tic/5 & 1)) -- Overtime?
	
	return flash, tics
end

FN.GetTimer = function(v, player)
-- As the name implies. Returns clock graphic to use and time (in string) to display.
-- Will also return raw time value (for other purposes) and flash boolean (to determine other behavior if needed).
	local patch, time = 'HUD_CLOCK'+tostring((leveltime%35)*235/1000), 0
	local flash, tics = FN.DoTimerLogic(player)
	
	local minutes = G_TicsToMinutes(tics, true)
	local seconds = G_TicsToSeconds(tics)
	local tictrn  = G_TicsToCentiseconds(tics)
	local spad, tpad = '', ''
	
	if (seconds < 10) then spad = '0' end	-- Number padding.
	if (tictrn < 10) then tpad = '0' end	-- v.drawString doesn't have that privilege.
	
	if (flash) then -- If not 'Overtime!' then draw normally. Otherwise flash!
		patch, time = 'HUD_RCLOCK'+tostring((leveltime%35)*235/1000), ''
		return v.cachePatch(patch), time, tics, flash
	end
	
	if (CV.TimerType.value == 2)
		-- Tics only. How simple is this?
		time = tics
	else
		if (CV.TimerType.value == 1 or modeattacking or marathonmode)
			-- Minutes, seconds and centiseconds. Down to the last detail.
			time = minutes+':'+spad+seconds+'.'+tpad+tictrn
		elseif (CV.TimerType.value == 0)
			-- Minutes and seconds. Your go-to classic.
			time = minutes+':'+spad+seconds
		end
	end
	
	return v.cachePatch(patch), time, tics, flash
end