local L = L_MinHud

local HUD = L.HeadsUpDisplay
local FN = L.Functions
local CV = L.Console

local FU = FRACUNIT
local minimal, classic, compact = 0, 1, 2

local tinyemeralds = {}
tinyemeralds[0] = "TEMER1"
tinyemeralds[1] = "TEMER2"
tinyemeralds[2] = "TEMER3"
tinyemeralds[3] = "TEMER4"
tinyemeralds[4] = "TEMER5"
tinyemeralds[5] = "TEMER6"
tinyemeralds[6] = "TEMER7"
local function DrawPowerstones(v, player, x, y, flags, xoffset, yoffset)
-- Draws the powerstones the player has collected so far (in Ringslinger).
	if (L.Tic&1)
	and	(player.powers[pw_invulnerability] and player.powers[pw_sneakers] == player.powers[pw_invulnerability])
	-- Hack. Extremely unlikely to be activated unintentionally.
	-- "Super" indicator!
		for j = 0, 6 do
			v.draw(x, y, v.cachePatch(tinyemeralds[j]), flags)
			x, y = $1 + xoffset, $2 + yoffset
		end
	else
	-- Powerstones.
		for j = 0, 6 do
			if player.powers[pw_emeralds] & (1 << j) then
				v.draw(x, y, v.cachePatch(tinyemeralds[j]), flags)
			end
			x, y = $1 + xoffset, $2 + yoffset
		end
	end
end

local function DrawWeaponRing(v, player, x, y)
-- Draw current weapon.
	local patch = FN.GetWeaponIcon(v, player, player.currentweapon)
	local ammo, maxammo, rawammo = FN.GetWeaponAmmo(player, player.currentweapon)
	
	local htrans, ptrans = V_30TRANS, V_70TRANS
	local flags1 = V_SNAPTOBOTTOM|V_PERPLAYER
	local flags2 = flags1
	
	if (player.currentweapon == 0) then
		if not (player.powers[pw_infinityring]) and not (player.rings) then
			flags1 = ptrans|$
		end
	else
		if FN.GotWeapon(player, 1<<(player.currentweapon-1)) and not (rawammo) then
			flags1 = htrans|$
		elseif not FN.GotWeapon(player, 1<<(player.currentweapon-1)) then
			flags1 = ptrans|$
		end
	end
	
	v.draw(x-8, y-8, patch, flags1)
	if (maxammo) then v.drawString(x+8, y, ammo, flags2, "thin-right") end
end

local function DrawWeaponSelect(v, player, x, y)
-- Draws the outline for our selected weapon.
-- Will also show the cooldown between each of our shots.
	local q, del, p = player.weapondelay, 0, 16
	while (q)
		if (q > p)
			del = $+p
			q = $-p
			q = $/2
			if (p > 1)
				p = $/2
			end
		else
			del = $+q
			break
		end
	end
	
	v.draw(x - 10, y - 10 - del/2, v.cachePatch("CURWEAP"), V_SNAPTOBOTTOM|V_PERPLAYER)
end

local function DrawWeaponInventory(v, player)
-- Draws the inventory of the player for weapon rings.
-- Displays their availability and ammunition.
	local x, y, flags = 4, 62, V_SNAPTOLEFT|V_PERPLAYER
	local sx, sy = 12, 63
	local htrans, ptrans = V_40TRANS, V_70TRANS
	
	for n = 0, 6 do
		local xsep, ysep = 0, 10*n
		
		local patch = FN.GetWeaponIcon(v, player, n)
		local ammo, maxammo, rawammo = FN.GetWeaponAmmo(player, n)
		
		if (n == 0)	or (n > 0) and FN.GotWeapon(player, 1<<(n-1)) then
			if (rawammo) then
				v.drawScaled((x+xsep)*FU, (y+ysep)*FU, FU/2, patch, flags)
				if (maxammo) then v.drawString(sx+xsep, sy+ysep, ammo, flags, "thin") end
			elseif not (rawammo) then
				v.drawScaled((x+xsep)*FU, (y+ysep)*FU, FU/2, patch, htrans|flags)
				if (maxammo) then v.drawString(sx+xsep, sy+ysep, ammo, htrans|flags, "thin") end
			end
		elseif (n > 0) and not FN.GotWeapon(player, 1<<(n-1)) then
			if (rawammo) then
				v.drawScaled((x+xsep)*FU, (y+ysep)*FU, FU/2, patch, ptrans|flags)
				v.drawString(sx+xsep, sy+ysep, ammo, ptrans|flags, "thin")
			end
		end
	end
end

local function DrawRingslingerHUD(v, player, x, y)
	if not G_RingSlingerGametype() then return end
	if G_TagGametype() and not (player.pflags & PF_TAGIT) then return end
	
	DrawWeaponRing(v, player, x, y)
	DrawWeaponSelect(v, player, x, y)
	DrawWeaponInventory(v, player)
	
	if (player.ammoremovaltimer) and (L.Tic%8 < 4) then
		v.drawString(160, 164, "-" + player.ammoremoval, V_REDMAP|V_SNAPTOBOTTOM, "center")
	end
end

HUD.MinHUDRingslinger = function(v, player)
-- Draw HUD related to ringslinger. This was more complicated then it had any right for.
	if FN.Draw() and not (player.spectator) then
		if (CV.Style.value == minimal) then
			DrawPowerstones(v, player, 4, 168, V_HUDTRANS|V_SNAPTOLEFT|V_SNAPTOBOTTOM|V_PERPLAYER, 8, 0)
			DrawRingslingerHUD(v, player, 160, 168)
		elseif (CV.Style.value == classic) then
			DrawPowerstones(v, player, 4, 172, V_HUDTRANS|V_SNAPTOLEFT|V_SNAPTOBOTTOM|V_PERPLAYER, 8, 0)
			DrawRingslingerHUD(v, player, 160, 188)
		elseif (CV.Style.value == compact) then
			DrawPowerstones(v, player, 308, 188, V_HUDTRANS|V_SNAPTORIGHT|V_SNAPTOBOTTOM|V_PERPLAYER, -8, 0)
			DrawRingslingerHUD(v, player, 12, 188)
		else
			return
		end
	end
end